function [stat] = statistics(cfg, varargin);

% STATISTICS_WRAPPER performs the selection of the biological data for
% timelock, frequency or source data and sets up the design vector or
% matrix.
%
% The specific configuration options for selecting timelock, frequency
% of source data are described in TIMELOCKSTATISTICS, FREQSTATISTICS and
% SOURCESTATISTICS, respectively.
% 
% After selecting the data, control is passed on to a data-independent
% statistical function that computes the probability of the
% null-hypothesis. The statistical subfunction that is called is
% STATISTICS_xxx, where cfg.approach='xxx'.
% 
% The specific configuration options for the statistical test are
% described in STATISTICS_xxx.

% Copyright (C) 2005, Robert Oostenveld
%
% $Log: statistics_wrapper.m,v $
% Revision 1.11  2005/10/11 13:13:35  roboos
% added avg to getsubfield(..., cfg.parameter) to be in line with trial
%
% Revision 1.10  2005/10/11 10:29:58  jansch
% removed usage of parameterselection which is redundant and might cause
% unnecessary crashes
%
% Revision 1.9  2005/10/05 08:01:41  roboos
% use parameterselection() subfunction for selecting source parameter
% added cfg.transform for log-transforming power
%
% Revision 1.8  2005/09/29 07:53:56  roboos
% fixed "if" bug caused by boolean operation on matrix comparison
%
% Revision 1.7  2005/08/19 17:01:32  roboos
% added check for identical transformation matrices
% changed selection of source volume, onbly inside voxels are used for computing statistics
%
% Revision 1.6  2005/06/10 07:14:22  roboos
% more carefull with selection of data from source input that may or may not be present in input (try-catch)
%
% Revision 1.5  2005/05/17 17:50:50  roboos
% changed all "if" occurences of & and | into && and ||
% this makes the code more compatible with Octave and also seems to be in closer correspondence with Matlab documentation on shortcircuited evaluation of sequential boolean constructs
%
% Revision 1.4  2005/04/28 07:22:09  roboos
% fixed typo in variable name (statfield_s_)
%
% Revision 1.3  2005/04/22 15:57:14  roboos
% added a check for the same dimensions and grid locations of the input source reconstructions
% extended the output, so that multiple statistical parameters (e.g. prob, F, beta) can be outputted in the same format
%
% Revision 1.2  2005/04/12 07:10:41  roboos
% added support for single trial source reconstructions, not yet checking on whether it is single trial or jacknife
%
% Revision 1.1  2005/04/06 14:26:51  roboos
% new implementation of a wrapper function. This is the glue between
% the data dependent interfaces for the end-user (sourcestatistics,
% timelockstatistics, freqstatistics) and the various back-end massive
% univariate statistical approaches that each is implemented in a separate
% function
%

% set  the defaults
if ~isfield(cfg, 'transform'), cfg.transform = []; end

% determine the type of input and output data
% functional (source) volumes have a pos field or a transform field
issourceinput  = isfield(varargin{1}, 'pos') || isfield(varargin{1}, 'transform');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% collect the biological data (the dependent parameter)
%  and the experimental design (the independent parameter)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if issourceinput
  % test that all source inputs have the same dimensions and are spatially aligned
  for i=2:length(varargin)
    if length(varargin{i}.dim)~=length(varargin{1}.dim) || ~all(varargin{i}.dim==varargin{1}.dim)
      error('dimensions of the source reconstructions do not match, use NORMALISEVOLUME first');
    end
    if isfield(varargin{1}, 'pos') && (length(varargin{i}.pos(:))~=length(varargin{1}.pos(:)) || ~all(varargin{i}.pos(:)==varargin{1}.pos(:)))
      error('grid locations of the source reconstructions do not match, use NORMALISEVOLUME first');
    end
    if isfield(varargin{1}, 'transform') && ~all(varargin{i}.transform(:)==varargin{1}.transform(:))
      error('spatial coordinates of the source reconstructions do not match, use NORMALISEVOLUME first');
    end
  end

  % get the source parameter on which the statistic should be evaluated
  %cfg.parameter = parameterselection(cfg.parameter, varargin{1});

  % get the source parameter data of each input source reconstruction
  Nsource = length(varargin);
  Nvoxel  = prod(varargin{1}.dim);
  if isfield(varargin{1}, 'trial')
    % get the single trial reconstructions
    for i=1:Nsource
       Ntrial(i) = length(varargin{i}.trial);
    end
    dat = zeros(Nvoxel, sum(Ntrial));
    k = 1;
    for i=1:Nsource
      for j=1:Ntrial(i)
        tmp = getsubfield(varargin{i}.trial(j), cfg.parameter);
        dat(:,k) = tmp(:);
        k = k+1;
      end
    end
  else
    % get the average source reconstructions
    dat = zeros(Nvoxel, Nsource);
    for i=1:Nsource
      tmp = getsubfield(varargin{i}.avg, cfg.parameter);
      dat(:,i) = tmp(:);
    end
  end
  design = cfg.design;
  if isfield(varargin{1}, 'inside')
    fprintf('only selecting voxels inside the brain for statistics (%.1f%%)\n', 100*length(varargin{1}.inside)/prod(varargin{1}.dim));
    dat = dat(varargin{1}.inside,:);
  end
else
  % get the ERF/TFR channel data, this can be done with a subfunction that is shared with CLUSTERRANDANALYSIS
  cfg.datarepresentation = 'concatenated';
  [cfg, data] = prepare_timefreq_data(cfg, varargin{:});
  if ~isfield(cfg, 'design')
    cfg.design = data.design(:,end)';
  end
  design = cfg.design;
  dim = size(data.biol);
  if length(dim)<3
    % add singleton frequency dimension
    dim(3) = 1;
  end
  if length(dim)<4
    % add singleton time dimension
    dim(4) = 1;
  end
  dat = reshape(data.biol, dim(1), prod(dim(2:end)))';
  % remove to save some memory
  data.biol = [];
end

% apply the optional transformation on the data, e.g. log(power)
if ~isempty(cfg.transform)
  fprintf('applying %s transformation on the parameter of interest\n', cfg.transform);
  dat = feval(cfg.transform, dat);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% compute the statistic, using the data-independent statistical subfunction
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
statfun = sprintf('statistics_%s', cfg.approach);
fprintf('evaluating %s\n', statfun);
stat = feval(statfun, cfg, dat, design);

if isstruct(stat)
  % the statistical output contains multiple elements, e.g. F-value, beta-weights and probability
  statfield = fieldnames(stat);
else
  % only the probability was returned as a single matrix, reformat into a structure
  stat.prob = stat;
  statfield = fieldnames(stat);
end

if issourceinput
  for i=1:length(statfield)
    tmp = getsubfield(stat, statfield{i});
    if isfield(varargin{1}, 'inside') && prod(size(tmp))==length(varargin{1}.inside)
      % the statistic was only computed on voxels that are inside the brain
      % sort the inside and outside voxels back into their original place
      tmp(varargin{1}.inside)  = tmp;
      tmp(varargin{1}.outside) = nan;
    end
    if prod(size(tmp))==prod(varargin{1}.dim)
      % reshape the statistical volumes into the original format
      stat = setsubfield(stat, statfield{i}, reshape(tmp, varargin{1}.dim));
    end
  end 
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% add descriptive information to the output and rehape into the input format
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if issourceinput
  % remember the definition of the volume, assume that they are identical for all input arguments
  try, stat.dim       = varargin{1}.dim;        end
  try, stat.xgrid     = varargin{1}.xgrid;      end
  try, stat.ygrid     = varargin{1}.ygrid;      end
  try, stat.zgrid     = varargin{1}.zgrid;      end
  try, stat.inside    = varargin{1}.inside;     end
  try, stat.outside   = varargin{1}.outside;    end
  try, stat.pos       = varargin{1}.pos;        end
  try, stat.transform = varargin{1}.transform;  end
else
  stat.dimord  = 'chan_freq_time';
  stat.label   = data.label;
  % remove dimensions that were not present in the original data
  if ~isnan(data.freq)
    stat.freq = data.freq;
  end
  if ~isnan(data.time)
    stat.time = data.time;
  end
  if isnan(data.time)
    stat.dimord = 'chan_freq';
    for i=1:length(statfield)
      tmp = getfield(stat, statfield{i});
      tmp = reshape(tmp, dim([2 3]));
      stat = setfield(stat, statfield{i}, tmp);
    end
  elseif isnan(data.freq)
    stat.dimord = 'chan_time';
    for i=1:length(statfield)
      tmp = getfield(stat, statfield{i});
      tmp = reshape(tmp, dim([2 4]));
      stat = setfield(stat, statfield{i}, tmp);
    end
  else
    stat.dimord = 'chan_freq_time';
    for i=1:length(statfield)
      tmp = getfield(stat, statfield{i});
      tmp = reshape(tmp, dim([2 3 4]));
      stat = setfield(stat, statfield{i}, tmp);
    end
  end
end

% add version information to the configuration
try
  % get the full name of the function
  cfg.version.name = mfilename('fullpath');
catch
  % required for compatibility with Matlab versions prior to release 13 (6.5)
  [st, i] = dbstack;
  cfg.version.name = st(i);
end
cfg.version.id = '$Id: statistics_wrapper.m,v 1.11 2005/10/11 13:13:35 roboos Exp $';

% remember the configuration of the input data
cfg.previous = [];
for i=1:length(varargin)
  if isfield(varargin{i}, 'cfg')
    cfg.previous{i} = varargin{i}.cfg;
  else
    cfg.previous{i} = [];
  end
end

% remember the exact configuration details
stat.cfg = cfg;

